; ------------------------------------------------------
; Empty project
; Copyright (c) 2005-2006, Franck Charlet
; All rights reserved.
; ------------------------------------------------------
; buildblock RELEASE
;	CAPT %5\Release.bat
; buildblockend
; buildblock DEBUG
;	CAPT %5\Debug.bat
; buildblockend

; ------------------------------------------------------
; Constants
SCREEN_WIDTH 		equ	800
SCREEN_HEIGHT		equ	600
SCREEN_BPP			equ	32

; ------------------------------------------------------
; Includes
			include	..\Common_Src\FrameWork.asm
			include ..\Common_Src\Fonts.asm
			include	..\Common_Src\Camera.asm
			
; ------------------------------------------------------
; Functions
Til_DrawMyCube		proto
Til_DrawDebugText	proto

; ------------------------------------------------------
; Variables
bDebug			dd	1
lFont			dd	0
deg_in_rad		dd	180

PosX			real4	0.0
PosY			real4	0.0
PosZ			real4	0.0
MoveShift		real4	0.1
LookShift		real4	0.5

My_Mouse_X		dd	0
My_Mouse_Y		dd	0
My_Mouse_Z		dd	0

Cam_Pitch		real4	0.0
Cam_Heading		real4	0.0

szPos_X		db	64 dup (0)
szPos_Y		db	64 dup (0)
szPos_Z		db	64 dup (0)
szMouse_X		db	64 dup (0)
szMouse_Y		db	64 dup (0)
szMouse_Z		db	64 dup (0)
szMouse_ButtonL		db	64 dup (0)
szMouse_ButtonR		db	64 dup (0)
szMouse_ButtonM		db	64 dup (0)
szCam_Heading		db	64 dup (0)
szCam_Pitch			db	64 dup (0)

; ------------------------------------------------------
; Name: Initialize()
; Desc: Perform the various initializations
Initialize		proc

			invoke	glClearDepth, CDBL(1.0)
			invoke	glHint, GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST
			invoke	glClearColor, 0, 0, 0, CFLT(1.0)
			invoke	glEnable, GL_NORMALIZE
			invoke	glEnable, GL_CULL_FACE
			
			; Create the fonts
			invoke	Create_Font_Bitmap, CSTR("Courier New"), 12, FW_NORMAL
			mov	lFont, eax
			.if	eax == 0
				invoke	Set_Error_Msg, CSTR("Can't create font")
				mov	eax, FALSE
				ret
			.endif

			mov	eax, TRUE
			ret
Initialize		endp

; ------------------------------------------------------
; Name: Deinitialize()
; Desc: Free allocated resources
Deinitialize		proc

			invoke Destroy_Font, lFont
			ret
Deinitialize		endp

; ------------------------------------------------------
; Name: Update()
; Desc: Update the scene
Update			proc	FrameRate:real4
			; Update scene here
			fild Mouse_X
			fmul LookShift
			fist Mouse_X
			fild Mouse_Y
			fmul LookShift
			fist Mouse_Y
			invoke	DInput_Clamp_Mouse, Mouse_X, addr My_Mouse_X, 0, SCREEN_WIDTH - 1
			invoke	DInput_Clamp_Mouse, Mouse_Y, addr My_Mouse_Y, -90, 90
			invoke	DInput_Clamp_Mouse, Mouse_Z, addr My_Mouse_Z, -1000, 1000
			
			fild	Mouse_X
			fadd	Cam_Heading
			fstp	Cam_Heading

			fild	My_Mouse_Y
			fstp	Cam_Pitch
			
			push	FLT2INT(Cam_Heading)
			push	CSTR("Cam_Heading: %d")
			push 	offset szCam_Heading
			invoke sprintf
			
			push	FLT2INT(Cam_Pitch)
			push	CSTR("Cam_Pitch: %d")
			push 	offset szCam_Pitch
			invoke sprintf
			
			push	FLT2INT(PosX)
			push	CSTR("Pos X: %d")
			push	offset szPos_X
			invoke	sprintf
			
			push	FLT2INT(PosY)
			push	CSTR("Pos Y: %d")
			push	offset szPos_Y
			invoke	sprintf
			
			push	FLT2INT(PosZ)
			push	CSTR("Pos Z: %d")
			push	offset szPos_Z
			invoke	sprintf
			
			push	My_Mouse_X
			push	CSTR("Mouse X: %d")
			push	offset szMouse_X
			invoke	sprintf

			push	My_Mouse_Y
			push	CSTR("Mouse Y: %d")
			push	offset szMouse_Y
			invoke	sprintf

			push	My_Mouse_Z
			push	CSTR("Mouse Z: %d")
			push	offset szMouse_Z
			invoke	sprintf

			movzx	eax, byte ptr Mouse_Left_Button
			push	eax
			push	CSTR("Left Button: %d")
			push	offset szMouse_ButtonL
			invoke	sprintf

			movzx	eax, byte ptr Mouse_Right_Button
			push	eax
			push	CSTR("Right Button: %d")
			push	offset szMouse_ButtonR
			invoke	sprintf

			movzx	eax, byte ptr Mouse_Middle_Button
			push	eax
			push	CSTR("Middle Button: %d")
			push	offset szMouse_ButtonM
			invoke	sprintf
			
			invoke  DInput_Get_Key, DIK_UP	; x += sin(heading), z += cos(heading)
			test  	eax, eax
			jz Up_Not_Pressed
				fld Cam_Heading ; in degrees, need radians! therefore mult. by pi/180
				fldpi			; 3.141...
				fmul
				fidiv deg_in_rad
				
				fsincos	; ST(0) = cos(heading), ST(1) = sin(heading)
				
				fmul MoveShift
				fadd PosZ
				fstp PosZ
				
				fmul MoveShift
				fsubr PosX
				fstp PosX
			Up_Not_Pressed:
			
			invoke  DInput_Get_Key, DIK_DOWN
			test  	eax, eax
			jz Down_Not_Pressed
				fld Cam_Heading ; in degrees, need radians! therefore mult. by pi/180
				fldpi			; 3.141...
				fmul
				fidiv deg_in_rad
				
				fsincos	; ST(0) = cos(heading), ST(1) = sin(heading)
				
				fmul MoveShift
				fsubr PosZ
				fstp PosZ
				
				fmul MoveShift
				fadd PosX
				fstp PosX
			Down_Not_Pressed:
			
			invoke  DInput_Get_Key, DIK_LEFT
			test  	eax, eax
			jz Left_Not_Pressed
				fld Cam_Heading ; in degrees, need radians! therefore mult. by pi/180
				fldpi			; 3.141...
				fmul
				fidiv deg_in_rad
				
				fsincos	; ST(0) = cos(heading), ST(1) = sin(heading)
				
				fmul MoveShift
				fadd PosX
				fstp PosX
				
				fmul MoveShift
				fadd PosZ
				fstp PosZ
			Left_Not_Pressed:
			
			invoke  DInput_Get_Key, DIK_RIGHT
			test  	eax, eax
			jz Right_Not_Pressed
				fld Cam_Heading ; in degrees, need radians! therefore mult. by pi/180
				fldpi			; 3.141...
				fmul
				fidiv deg_in_rad
				
				fsincos	; ST(0) = cos(heading), ST(1) = sin(heading)
				
				fmul MoveShift
				fsubr PosX
				fstp PosX
				
				fmul MoveShift
				fsubr PosZ
				fstp PosZ
			Right_Not_Pressed:
			
			invoke	DInput_Get_Key, DIK_ESCAPE
			test	eax, eax
			jz	Esc_Not_Pressed
				invoke	Terminate_Application
			Esc_Not_Pressed:
			
			ret
Update			endp

; ------------------------------------------------------
; Name: Draw()
; Desc: Draw the scene
Draw			proc
			invoke	glClear, GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT or GL_STENCIL_BUFFER_BIT

			; Draw scene here
			mov eax, bDebug
			test eax, eax
			jz NoDebug
				invoke Til_DrawDebugText
			NoDebug:

			invoke	glPushMatrix
				invoke	Cam_Orientation, Cam_Pitch, Cam_Heading
				
				invoke	glBegin, GL_QUADS
					invoke	glColor3f, CFLT(0.5333), CFLT(0.0), CFLT(0.0)
					invoke	glVertex3f, CFLT(-100.0), CFLT(-1.01), CFLT(-100.0)
					invoke	glVertex3f, CFLT(-100.0), CFLT(-1.01), CFLT(100.0)
					invoke	glVertex3f, CFLT(100.0), CFLT(-1.01), CFLT(100.0)
					invoke	glVertex3f, CFLT(100.0), CFLT(-1.01), CFLT(-100.0)
				invoke	glEnd
				
				invoke  glTranslatef, PosX, PosY, PosZ
				invoke	glTranslatef, CFLT(-2.0), CFLT(0.0), CFLT(-4.0)
				
				invoke	Til_DrawMyCube
				
				invoke	glTranslatef, CFLT(4.0), CFLT(0.0), CFLT(0.0)
				
				invoke	Til_DrawMyCube
				
				;invoke	glTranslatef, CFLT(0.0), CFLT(0.0), CFLT(0.0)
				;invoke	Display_3D_Text, lFont, CSTR("COLLISION"), 9
			invoke	glPopMatrix
			ret
Draw			endp

; ------------------------------------------------------
; Name: Til_DrawMyCube()
; Desc: Draw a cube relative to 0,0,0
Til_DrawMyCube		proc
	LOCAL red 		:real4
	LOCAL green		:real4
	LOCAL blue 		:real4
	LOCAL max_z		:word
		
	mov max_z, 1000
	
	fild My_Mouse_Z
	fabs
	fidiv max_z
	fst red
	fst blue
	fstp green
	
	invoke	glBegin, GL_QUADS
		;invoke	glColor3f, CFLT(0.0), CFLT(0.2), CFLT(0.3)
		;invoke	glColor3f, red, blue, green
		invoke	glColor3f, red, CFLT(0.2), CFLT(0.3)
		invoke	glVertex3f, CFLT(1.0), CFLT(1.0), CFLT(-1.0)
		invoke	glVertex3f,CFLT(-1.0), CFLT(1.0), CFLT(-1.0)
		invoke	glVertex3f,CFLT(-1.0), CFLT(1.0), CFLT(1.0)
		invoke	glVertex3f, CFLT(1.0), CFLT(1.0), CFLT(1.0)

		invoke	glVertex3f, CFLT(1.0),CFLT(-1.0), CFLT(1.0)
		invoke	glVertex3f,CFLT(-1.0),CFLT(-1.0), CFLT(1.0)
		invoke	glVertex3f,CFLT(-1.0),CFLT(-1.0),CFLT(-1.0)
		invoke	glVertex3f, CFLT(1.0),CFLT(-1.0),CFLT(-1.0)

		;invoke	glColor3f, CFLT(0.2), CFLT(0.4), CFLT(0.5)
		invoke	glColor3f, CFLT(0.2), blue, CFLT(0.5)
		invoke	glVertex3f, CFLT(1.0), CFLT(1.0), CFLT(1.0)
		invoke	glVertex3f,CFLT(-1.0), CFLT(1.0), CFLT(1.0)
		invoke	glVertex3f,CFLT(-1.0),CFLT(-1.0), CFLT(1.0)
		invoke	glVertex3f, CFLT(1.0),CFLT(-1.0), CFLT(1.0)

		invoke	glVertex3f, CFLT(1.0),CFLT(-1.0),CFLT(-1.0)
		invoke	glVertex3f,CFLT(-1.0),CFLT(-1.0),CFLT(-1.0)
		invoke	glVertex3f,CFLT(-1.0), CFLT(1.0),CFLT(-1.0)
		invoke	glVertex3f, CFLT(1.0), CFLT(1.0),CFLT(-1.0)

		;invoke	glColor3f, CFLT(0.4), CFLT(0.6), CFLT(0.7)
		invoke	glColor3f, CFLT(0.4), CFLT(0.6), green
		invoke	glVertex3f, CFLT(-1.0), CFLT(1.0), CFLT(1.0)
		invoke	glVertex3f, CFLT(-1.0), CFLT(1.0),CFLT(-1.0)
		invoke	glVertex3f, CFLT(-1.0),CFLT(-1.0),CFLT(-1.0)
		invoke	glVertex3f, CFLT(-1.0),CFLT(-1.0), CFLT(1.0)

		invoke	glVertex3f, CFLT(1.0), CFLT(1.0),CFLT(-1.0)
		invoke	glVertex3f, CFLT(1.0),CFLT(1.0), CFLT(1.0)
		invoke	glVertex3f, CFLT(1.0),CFLT(-1.0), CFLT(1.0)
		invoke	glVertex3f, CFLT(1.0),CFLT(-1.0),CFLT(-1.0)
	invoke	glEnd
	
	ret
Til_DrawMyCube		endp

; ------------------------------------------------------
; Name: Til_DrawDebugText()
; Desc: Draws helpful debug values to top-left of window
Til_DrawDebugText		proc

	invoke	Enter_2D_Mode, INT2FLT(SCREEN_WIDTH), INT2FLT(SCREEN_HEIGHT)
		invoke	glPushMatrix
		invoke	glTranslatef, CFLT(0.0), CFLT(0.0), CFLT(-1.0)
		invoke	glColor3f, CFLT(0.4), CFLT(0.8), CFLT(0.3)
		
		lea	ebx, szPos_X
		invoke	lstrlen, ebx
		invoke	Display_2D_Text, INT2FLT(0), INT2FLT(10), lFont, ebx, eax
		lea	ebx, szPos_Y
		invoke	lstrlen, ebx
		invoke	Display_2D_Text, INT2FLT(0), INT2FLT(22), lFont, ebx, eax
		lea	ebx, szPos_Z
		invoke	lstrlen, ebx
		invoke	Display_2D_Text, INT2FLT(0), INT2FLT(34), lFont, ebx, eax
		lea	ebx, szMouse_X
		invoke	lstrlen, ebx
		invoke	Display_2D_Text, INT2FLT(0), INT2FLT(46), lFont, ebx, eax
		lea	ebx, szMouse_Y
		invoke	lstrlen, ebx
		invoke	Display_2D_Text, INT2FLT(0), INT2FLT(58), lFont, ebx, eax
		lea	ebx, szMouse_Z
		invoke	lstrlen, ebx
		invoke	Display_2D_Text, INT2FLT(0), INT2FLT(70), lFont, ebx, eax
		lea	ebx, szMouse_ButtonL
		invoke	lstrlen, ebx
		invoke	Display_2D_Text, INT2FLT(0), INT2FLT(82), lFont, ebx, eax
		lea	ebx, szMouse_ButtonR
		invoke	lstrlen, ebx
		invoke	Display_2D_Text, INT2FLT(0), INT2FLT(94), lFont, ebx, eax
		lea	ebx, szMouse_ButtonM
		invoke	lstrlen, ebx
		invoke	Display_2D_Text, INT2FLT(0), INT2FLT(106), lFont, ebx, eax
		lea	ebx, szCam_Heading
		invoke	lstrlen, ebx
		invoke	Display_2D_Text, INT2FLT(0), INT2FLT(118), lFont, ebx, eax
		lea	ebx, szCam_Pitch
		invoke	lstrlen, ebx
		invoke	Display_2D_Text, INT2FLT(0), INT2FLT(130), lFont, ebx, eax
		
		invoke	glPopMatrix
	invoke	Leave_2d_Mode

	ret
Til_DrawDebugText		endp

end start